# Copyright 2012-2013 GRNET S.A. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
#
#   1. Redistributions of source code must retain the above
#      copyright notice, this list of conditions and the following
#      disclaimer.
#
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials
#      provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY GRNET S.A. ``AS IS'' AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GRNET S.A OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and
# documentation are those of the authors and should not be
# interpreted as representing official policies, either expressed
# or implied, of GRNET S.A.

from django.db import transaction
from django.db import connection
from time import sleep
import logging

logger = logging.getLogger(__name__)


def with_lock(retries=3, retry_wait=1.0):
    def wrap(func):
        def inner(*args, **kwargs):

            transaction.commit()

            _retries = retries
            cursor = connection.cursor()
            locked = True
            try:
                while 1:
                    cursor.execute("SELECT pg_try_advisory_lock(1)")
                    r = cursor.fetchone()
                    if r is None:
                        m = "Impossible"
                        raise AssertionError(m)
                    locked = r[0]
                    if locked:
                        break

                    _retries -= 1
                    if _retries <= 0:
                        return False
                    sleep(retry_wait)

                return func(*args, **kwargs)

            finally:
                if locked:
                    try:
                        transaction.commit()
                    except Exception as e:
                        logger.exception(e)
                        transaction.rollback()

                    cursor.execute("SELECT pg_advisory_unlock(1)")
                    cursor.fetchall()
        return inner
    return wrap
